% k nearest neighbors for points in two dimensions. With k = 1, we can use
% a point-location algorithm on the plane, based on a Delaunay
% triangulation. With other values of k, we use the inefficient, but easy
% to code method. This could be sped up for instance with kd trees.
% The array point contains the P points to be classified, arranged in a 2 x
% P array. The 2 x S array sample contains the S sample points, and the 1 x
% S vector label contains the labels (either 0 or 1) of the sample points.
% The parameter k (default 1 if not specified) is the number of neighbors
% to use for classification (the label is the one that achieves the
% absolute majority of k votes, so you want to make k odd).

function c = knn(point, sample, label, k)

if nargin < 4
    k = 1;
end

P = size(point, 2);
c = zeros(1, P);

if k == 1
    tri = delaunay(sample(1, :), sample(2, :));
    index = dsearchn(sample', tri, point');
    c(label(index) == 1) = 1;
else
    S = size(sample, 2);
    sx = sample(1, :);
    sy = sample(2, :);
    thr = k/2;
    for p = 1:P
        d = (sx - point(1, p)) .^ 2 + (sy - point(2, p)) .^ 2;
        [d, index] = sort(d);
        index = index(1:k);
        c(p) = sum(label(index) == 1) > thr;
    end
end
c = logical(c);
